unit RTDesign;

(*******************************************************************************
                      Run-time Designer
               Copyright  Karl-Otto Rosenqvist
********************************************************************************

2001-10-16: Marco Antonio Soto Rodriguez Mexico
            Memoryleaks in CopyToClipboard and PasteFromClipboard

*******************************************************************************)





interface

uses SysUtils, WinTypes, WinProcs, Classes, Controls, Messages, Forms,
     Menus, Graphics;

Type
  THandleKind = (hkNW, hkN, hkNE, hkE, hkSE, hkS, hkSW, hkW);

  TApplySizeEvent = Procedure(Sender : TObject;SizedControl : TControl;oldBounds, newBounds : TRect;
      var ApplySize : Boolean) of Object;
  TBaseCtlClickEvent = Procedure(Sender : TObject;var ClearSelection, DrawSelectionFrame : Boolean;
    Shift : TShiftState;X, Y : Integer) of Object;
  TRTDDblClick = Procedure(Sender : TObject;Control : TControl) of Object;
  TSelectControlEvent = Procedure(Sender : TObject;Control : TControl;var DoSelect : Boolean) of Object;
  TDeleteQueryEvent = Procedure(Sender : TObject;var DoDelete : Boolean) of Object;
  TInsertEvent = Procedure(Sender : TObject;Control : TControl;var aClass : TControlClass) of Object;
  TRemoveNotifyEvent = Procedure(Sender : TObject;aControl : TControl) of Object;
  TDeletedNotifyEvent = Procedure(Sender : TObject) of Object;
  TAfterInsertEvent = Procedure(Sender : TObject;Control : TControl) of Object;

  TRTDMouseEvent = Procedure(Sender : TObject;Control : TControl;Button : TMouseButton;
      Shift : TShiftState;X, Y : Integer) of Object;
  TRTDMouseMove = Procedure(Sender : TObject;Control : TControl;Shift : TShiftState;
      X, Y : Integer;BaseControlCoords : TPoint) of Object;


  TRTDOption = (rtdProportionalSizing, rtdCenterSizing, rtdPropertiesMenuItem,
      rtdSnapToGrid);
  TRTDOptions = set of TRTDOption;

  TGrabHandleKind = (gkHighContrast, gkDelphi, gk3D);

  TMouseStatus = (msNone, msMoving, msSizing, msFraming);

  TUndoKind   = (ukNone, ukMove, ukSize, ukPropertiesChanged, ukDelete, ukInsert);

const
  CUndoString : Array[ukNone..ukInsert] of String =
    ('', 'Move', 'Size', 'Properties changed', 'Delete', 'Insert');

Type
  TKeySupportOptions = (ksoNotActive, ksoActive, ksoCtrl, ksoAlt, ksoCtrlAlt);
  TKeySupportObject = class(TPersistent)
  Private
    FCopy : TKeySupportOptions;
    FCut : TKeySupportOptions;
    FDelete : TKeySupportOptions;
    FInsert : TKeySupportOptions;
    FPaste : TKeySupportOptions;
    FSelectAll : TKeySupportOptions;
    FSelectNone : TKeySupportOptions;
    FUndo : TKeySupportOptions;
  Public
    Constructor Create; 
  Published
    Property A : TKeySupportOptions Read FSelectAll Write FSelectAll;
    Property C : TKeySupportOptions Read FCopy Write FCopy;
    Property Del : TKeySupportOptions Read FDelete Write FDelete;
    Property Ins : TKeySupportOptions Read FInsert Write FInsert;
    Property N : TKeySupportOptions Read FSelectNone Write FSelectNone;
    Property V : TKeySupportOptions Read FPaste Write FPaste;
    Property X : TKeySupportOptions Read FCut Write FCut;
    Property Z : TKeySupportOptions Read FUndo Write FUndo;
  End;

  TUndoObject = class(TObject)
  Private
    FComps : TStringList;
    FKind : TUndoKind;
    FStream : TMemoryStream;
  Public
    Constructor Create(aKind : TUndoKind);
    Destructor Destroy; override;
    Property Components : TStringList Read FComps;
    Property Kind : TUndoKind Read FKind Write FKind;
    Property Stream : TMemoryStream Read FStream;
  End;

  TDesignerStates = (dsCreating, dsLoading, dsDestroying);
  TDesignerState = set of TDesignerStates;

  TRTDesigner = class;
  TGrabHandle = class(TCustomControl)
  Private
    FControl : TControl;
    FKind : THandleKind;
    MyDesigner : TRTDesigner;
    Procedure SetControl(aControl : TControl);
    Procedure SetKind(aKind : THandleKind);
  Protected
    Procedure Paint; override;
  Public
    Constructor CreateNew(aOwner : TRTDesigner;aParent : TWinControl;
          aControl : TControl;aKind : THandleKind);
    destructor Destroy; override;
    Procedure SetPosition;
    Property Control : TControl Read FControl Write SetControl;
    Property Kind : THandleKind Read FKind Write SetKind;
  End;

  TPopulateMenuEvent = Procedure(Sender : TObject;var Menu : TPopupMenu) of Object;
  TGetPopupMenuEvent = procedure(Sender : TObject;var Menu : TPopupMenu) of Object;
  TCheckIfDesigningEvent = procedure(Sender, aObject : TObject;X, Y : Integer;var IsDesigning : Boolean) of Object;

  TRTDesigner = class(TComponent)
  Private
    _handlescreated : Boolean;
    FAcceptedControls : TList;
    FBaseControl : TWinControl;
    FClipControl : TControl;
    FControls : TList;
    FDesignCursor : TCursor;
    FDesignPop : TPopupMenu;
    FDesignerState : TDesignerState;
    FDesigning : Boolean;
    FDown, FLast : TPoint;
    FEditMode : Boolean;
    FGrabbedHandle : THandleKind;
    FGrabHandleKind : TGrabHandleKind;
    FGridX, FGridY : Integer;
    FHandles : Array[0..7] of TGrabHandle;
    FKeySupport : TKeySupportObject;
    FMouseControl : TControl;
    FMouseStatus : TMouseStatus;
    FOptions : TRTDOptions;
    FReadErrors : TList;
    FShowGrabHandles: Boolean;
    FShowMultiSelection : Boolean;
    FSizingRect : TRect;
    FUndoLimit : Byte;
    FUndoList : TList;
    MyForm : TForm;

    FAfterBCClear : TNotifyEvent;
    FAfterInsert : TAfterInsertEvent;
    FAfterSized : TNotifyEvent;
    FOnApplySize : TApplySizeEvent;
    FOnBaseClick : TBaseCtlClickEvent;
    FOnCheckIfDesigning : TCheckIfDesigningEvent;
    FOnControlDblClick : TRTDDblClick;
    FOnDelete : TDeleteQueryEvent;
    FOnDeletedControl : TDeletedNotifyEvent;
    FOnEditModeChange : TNotifyEvent;
    FOnInsert : TInsertEvent;
    FOnGetPopupMenu : TGetPopupMenuEvent;
    FOnKeyDown : TKeyEvent;
    FOnKeyUp : TKeyEvent;
    FOnMouseDown : TRTDMouseEvent;
    FOnMouseMove : TRTDMouseMove;
    FOnMouseUp : TRTDMouseEvent;
    FOnMoved : TNotifyEvent;
    FOnPopulateMenu : TPopulateMenuEvent;
    FOnRemoveControl : TRemoveNotifyEvent;
    FOnSelectControl : TSelectControlEvent;
    FOnSelectionChanged : TNotifyEvent;
    FOnUndoChange : TNotifyEvent;
    Procedure ActivateApp;
    Procedure ApplicationMessages(var Msg: TMsg; var Handled: Boolean);
    Function GetControlCount : Integer;
    Function ControlOnBaseControl(aControl : TControl) : Boolean;
    Procedure CreateHandles;
    Procedure DeactivateApp;
    Procedure DestroyHandles;
    Procedure DrawFrame(Handle : THandle;R : TRect);
    Procedure DrawMovingFrames;
    Function DrawSelectFrame : TRect;
    Procedure DrawSizingFrame;
    Function GetActive : Boolean;
    Function GetControl(index : Integer) : TControl;
    Function GetUndoKind : TUndoKind;
    Function GetUndoString : String;
    Function GetVer : String;
    Procedure LimitCursorTo(aControl : TControl); { by Sergey Mishkovskiy }
    Procedure PaintHandlesOnCanvas(aControl : TControl);
    Procedure PaintSelection;
    Procedure ReaderSetName(Reader: TReader; Component: TComponent; var Name: string);
    Procedure ReadComponentsProc(Component: TComponent);
    Procedure RemoveUnnecessaryControlsFromSelection;
    Procedure RepaintControl(aCtrl : TControl;Inflated : Boolean);
    Procedure SelectControlsInRect(aRect : TRect;AllControls : Boolean);
    Procedure SelectNewControl(aControl : TControl);
    Procedure SetEditMode(Value : Boolean);
    Function ControlAspect(aControl : TControl) : Extended;
    Procedure RemoveFromSelection(aControl : TControl);
    Procedure SetBaseControl(aControl : TWinControl);
    Procedure SetVer(const A : String);
  Protected
    Function AssumedOwner : TWinControl;
    Function BaseControlParentForm : TForm;
    Function DoInsertEvent(aParent : TControl;X, Y : Integer) : Boolean;
    Procedure Loaded; override;
    Procedure PopAlignToGrid(Sender : TObject);
    Procedure PopBringToFront(Sender : TObject);
    Procedure PopSendToBack(Sender : TObject);
    Procedure PopulateDesignPopup; virtual;
    procedure SelectionChanged; virtual;
    Property GrabbedHandle : THandleKind Read FGrabbedHandle;
    Property MouseStatus : TMouseStatus Read FMouseStatus;
  Public
    Constructor Create(aOwner : TComponent); override;
    Destructor Destroy; override;
    Function AddControl(aClass : TControlClass;aParent : TWinControl;X, Y : Integer;
      aName : String;aVisible : Boolean) : TControl;
    Procedure AddToSelection(aControl : TControl);
    Procedure AddUndo(Kind : TUndoKind);
    Procedure AlignToGrid;
    Procedure BringToFront;
    Procedure ClearBaseControl;
    Procedure CopyToClipboard;
    Procedure DeleteSelection;
    Function IsGrabHandle(aControl : TControl) : Boolean;
    Procedure LoadBCFromFile(const aFileName : String);
    Procedure LoadBCFromStream(Stream : TStream);
    Procedure MoveLeft;
    Procedure MoveRight;
    Procedure MoveUp;
    Procedure MoveDown;
    Procedure Notification(aComponent : TComponent;Operation : TOperation); override;
    Procedure PasteFromClipboard;
    Procedure RefreshHandlesPos;
    procedure RemoveOtherRTDesigners;
    Procedure ResetUndo;
    Procedure ResizeLeft;
    Procedure ResizeRight;
    Procedure ResizeUp;
    Procedure ResizeDown;
    Procedure SaveBCToFile(const aFileName : String);
    Procedure SaveBCToStream(Stream : TStream);
    Procedure SelectAll;
    Function Selected(aControl : TControl): Boolean;
    Procedure SelectNone;
    Procedure SendToBack;
    Procedure ShowSelection(Show : Boolean);
    Procedure Undo;
    Procedure UpdateSelection;
    Property AcceptedControls : TList Read FAcceptedControls;
    Property Active : Boolean Read GetActive;
    Property Control[index : Integer] : TControl Read GetControl;
    Property ControlCount : Integer Read GetControlCount;
    Property Controls : TList Read FControls;
    property Designing : Boolean read FDesigning;
    Property UndoKind : TUndoKind Read GetUndoKind;
    Property UndoString : String Read GetUndoString;
  Published
    Property BaseControl : TWinControl Read FBaseControl Write SetBaseControl;
    property DesignCursor : TCursor read FDesignCursor write FDesignCursor;
    Property EditMode : Boolean Read FEditMode Write SetEditMode;
    Property GrabHandleKind : TGrabHandleKind Read FGrabHandleKind Write FGrabHandleKind;
    Property GridX : Integer Read FGridX Write FGridX;
    Property GridY : Integer Read FGridY Write FGridY;
    Property KeySupportOptions : TKeySupportObject Read FKeySupport Write FKeySupport;
    Property Options : TRTDOptions Read FOptions Write FOptions;
    property ShowGrabHandles : Boolean read FShowGrabHandles write FShowGrabHandles default True;
    property ShowMultiSelection : Boolean read FShowMultiSelection write FShowMultiSelection default True;
    Property UndoLimit : Byte Read FUndoLimit Write FUndoLimit;
    Property Version : String Read GetVer Write SetVer;
    Property AfterBaseControlClear : TNotifyEvent Read FAfterBCClear Write FAfterBCClear;
    Property OnAfterDeleteControl : TDeletedNotifyEvent Read FOnDeletedControl Write FOnDeletedControl;
    Property AfterInsert : TAfterInsertEvent Read FAfterInsert Write FAfterInsert;
    Property AfterMove : TNotifyEvent Read FOnMoved Write FOnMoved;
    Property AfterSized : TNotifyEvent Read FAfterSized Write FAfterSized;
    Property OnApplySize : TApplySizeEvent Read FOnApplySize Write FOnApplySize;
    Property OnDeleteQuery : TDeleteQueryEvent Read FOnDelete Write FOnDelete;
    Property BeforeSelectControl : TSelectControlEvent  Read FOnSelectControl Write FOnSelectControl;
    Property OnBaseControlClick : TBaseCtlClickEvent Read FOnBaseClick Write FOnBaseClick;
    property OnCheckIfDesigning : TCheckIfDesigningEvent read FOnCheckIfDesigning write FOnCheckIfDesigning;
    Property OnDblClick : TRTDDblClick Read FOnControlDblClick Write FOnControlDblClick;
    Property OnEditModeChange : TNotifyEvent Read FOnEditModeChange Write FOnEditModeChange;
    property OnGetPopupMenu : TGetPopupMenuEvent read FOnGetPopupMenu write FOnGetPopupMenu;
    Property OnInsertQuery : TInsertEvent Read FOnInsert Write FOnInsert;
    Property OnKeyDown : TKeyEvent Read FOnKeyDown Write FOnKeyDown;
    Property OnKeyUp : TKeyEvent Read FOnKeyUp Write FOnKeyUp;
    Property OnMouseDown : TRTDMouseEvent Read FOnMouseDown Write FOnMouseDown;
    Property OnMouseMove : TRTDMouseMove Read FOnMouseMove Write FOnMouseMove;
    Property OnMouseUp : TRTDMouseEvent Read FOnMouseUp Write FOnMouseUp;
    Property OnPopulateDesignerMenu : TPopulateMenuEvent Read FOnPopulateMenu Write FOnPopulateMenu;
    Property OnRemoveControl : TRemoveNotifyEvent Read FOnRemoveControl Write FOnRemoveControl;
    property OnSelectionChanged : TNotifyEvent read FOnSelectionChanged write FOnSelectionChanged;
    Property OnUndoChange : TNotifyEvent Read FOnUndoChange Write FOnUndoChange;
  End;

  TDesignServer = class(TComponent)
  Private
    FDesigners : TList;
    FOnActivate : TNotifyEvent;
    FOnDeactivate : TNotifyEvent;
    FOnMessage : TMessageEvent;
    Procedure ActivateApp(Sender : TObject);
    Procedure ApplicationMessage(var Msg: TMsg; var Handled: Boolean);
    Procedure DeactivateApp(Sender : TObject);
  Public
    Constructor Create(aOwner : TComponent); override;
    Destructor Destroy; override;
    Procedure AddDesigner(aDesigner : TRTDesigner);
    Procedure RemoveDesigner(aDesigner : TRTDesigner);
  Published
    Property OnActivate : TNotifyEvent Read FOnActivate Write FOnActivate;
    Property OnDeactivate : TNotifyEvent Read FOnDeactivate Write FOnDeactivate;
    Property OnMessage : TMessageEvent Read FOnMessage Write FOnMessage;
  End;

function GetControlUnderMouse : TControl;
Function RuntimeDesignerVersionStr : String;
Procedure Register;

var
  DesignServer : TDesignServer;


implementation

uses Dialogs,
{$ifdef Win32}
     Registry,
{$else}
    IniFiles,
{$endif}
     Clipbrd, RTDReg, RTDCnsts, ExtCtrls;

const
  verMajor   = 2;
  verMinor   = 1;
  verRelease = 12;
  verBuild   = 32;
{$ifdef ver80}
  ClipboardTextBufSize = 32767; { 32 kB}
{$else}
  ClipboardTextBufSize = 10*1024*1024; { 10 MB }
{$endif}
  HandleWidth  = 5;
  HandleWidthHalf = HandleWidth div 2;


function GetControlUnderMouse : TControl;
var P : TPoint;

  function MouseOverChild(aControl : TWinControl) : TControl;
  var i : Integer;
      aRect: TRect;
  begin
    Result:=aControl;
    for i:=aControl.ControlCount - 1 downto 0 do begin
      with aControl.Controls[i] do begin
        with ClientToScreen(Point(0, 0)) do
          aRect:=Bounds(x, y, Width, Height);
        if PtInRect(aRect, P) then begin
          Result:=aControl.Controls[i];
          if aControl.Controls[i].InheritsFrom(TWinControl) then
            Result:=MouseOverChild(TWinControl(Result));
          Break;
        end;
      end;
    end;
  end;


begin
  GetCursorPos(P);
  Result:=FindDragTarget(P, True);
  if (Result<>nil) and (Result.InheritsFrom(TWinControl)) then
    Result:=MouseOverChild(TWinControl(Result));
end;


Function RuntimeDesignerVersionStr : String;
Begin
  Result:=Format('%d.%d.%d.%d', [verMajor, verMinor, verRelease, verBuild]);
End;

Function ClientRectToScreen(aCtrl : TControl) : TRect;
Begin
  Result.TopLeft:=aCtrl.ClientToScreen(Point(0, 0));
  Result.BottomRight:=aCtrl.ClientToScreen(Point(aCtrl.Width, aCtrl.Height));
End;


Procedure Register;
var Show : Boolean;
Begin
  RegisterComponents('Additional', [TRTDesigner]);
{$ifdef Win32}
  With TRegIniFile.Create('Software\Towebo\Run-time Designer') Do Try
    Show:=not ReadBool('2.0', 'Installed', False);
    WriteBool('2.0', 'Installed', True);
  Finally
    Free;
  End;
{$else}
  With TIniFile.Create('Delphi.ini') Do Try
    Show:=not ReadBool('Run-time Designer', 'Installed', False);
    WriteBool('Run-time Designer', 'Installed', True);
  Finally
    Free;
  End;
{$endif}
  if Show Then MessageDlg('Congratulations! You have just installed Run-time Designer.'#13#10+
    'Visit http://welcome.to/towebo for updates and information.', mtInformation, [mbOk], 0);
End;

Function GetHandleRect(aRect : TRect;aHandle : THandleKind) : TRect;
var halfW, halfH : Integer;
Begin
  With aRect Do Begin
    halfW:=(Right-Left) div 2;
    halfH:=(Bottom-Top) div 2;
    Result:=Bounds(0, 0, 5, 5);
    Case aHandle of
      hkNW : OffsetRect(Result, Left, Top);
      hkN  : OffsetRect(Result, Left+halfW-HandleWidthHalf, Top);
      hkNE : OffsetRect(Result, Right-HandleWidth, Top);
      hkE  : OffsetRect(Result, Right-HandleWidth, Top+halfH-HandleWidthHalf+1);
      hkSE : OffsetRect(Result, Right-HandleWidth, Bottom-HandleWidth);
      hkS  : OffsetRect(Result, Left+halfW-HandleWidthHalf, Bottom-HandleWidth);
      hkSW : OffsetRect(Result, Left, Bottom-HandleWidth);
      hkW  : OffsetRect(Result, Left, Top+halfH-HandleWidthHalf+1);
    End;
  End;
End;

Function IsMouseDownMessage(Msg : Integer) : Boolean;
Begin
  Case Msg of
    WM_LBUTTONDOWN, WM_LBUTTONDBLCLK,
    WM_MBUTTONDOWN, WM_MBUTTONDBLCLK,
    WM_RBUTTONDOWN, WM_RBUTTONDBLCLK,
    WM_NCLBUTTONDOWN, WM_NCMBUTTONDOWN, WM_NCRBUTTONDOWN : Result:=True;
    Else Result:=False;
  End;
End;


{ TKeySupportObject ------------------------------------------------------------}
Constructor TKeySupportObject.Create;
Begin
  inherited Create;
  FCopy:=ksoCtrl;
  FCut:=ksoCtrl;
  FDelete:=ksoActive;
  FInsert:=ksoActive;
  FPaste:=ksoCtrl;
  FSelectAll:=ksoCtrl;
  FSelectNone:=ksoCtrl;
  FUndo:=ksoCtrl;
End;


{ TUndoObject ------------------------------------------------------------------------}
Constructor TUndoObject.Create(aKind : TUndoKind);
Begin
  inherited Create;
  FComps:=TStringList.Create;
  FKind:=aKind;
  FStream:=TMemoryStream.Create;
End;

Destructor TUndoObject.Destroy;
Begin
  FComps.Free;
  FStream.Free;
  inherited Destroy;
End;


{ TGrabHandle --------------------------------------------------------------------}
Constructor TGrabHandle.CreateNew(aOwner : TRTDesigner;aParent : TWinControl;
          aControl : TControl;aKind : THandleKind);
Begin
  inherited Create(aOwner);
  MyDesigner:=aOwner;
  Parent:=aParent;
  FControl:=aControl;
  Kind:=aKind;
  Width:=HandleWidth;
  Height:=HandleWidth;
  Hide;
End;

destructor TGrabHandle.Destroy;
var i : Integer;
begin
  if Assigned(Owner) and Owner.InheritsFrom(TRTDesigner) then
  with Owner as TRTDesigner do
  begin
    _handlescreated:=False;
    for i:=0 to 7 do
    begin
      if FHandles[i] = Self then
        FHandles[i]:=nil;
    end;
  end;
  inherited;
end;

Procedure TGrabHandle.Paint;
var aRect : TRect;
    colHighlight, colShadow, colFace : TColor;
Begin
  Case MyDesigner.GrabHandleKind of
    gkHighContrast : Begin
      colHighlight:=clBlack;
      colShadow:=clBlack;
      colFace:=clWhite;
    End;
    gkDelphi : Begin
      colHighlight:=clBlack;
      colShadow:=clBlack;
      colFace:=clBlack;
    End;
    gk3D : Begin
      colHighlight:=clBtnHighlight;
      colShadow:=clBtnShadow;
      colFace:=clBtnFace;
    End;
    Else Begin
      colHighlight:=clBlack;
      colShadow:=clBlack;
      colFace:=clWhite;
    End;
  End;
  aRect:=ClientRect;
  Canvas.Brush.Color:=colFace;
  Canvas.FillRect(aRect);
  Frame3D(Canvas, aRect, colHighlight, colShadow, 1);
End;

Procedure TGrabHandle.SetControl(aControl : TControl);
Begin
  FControl:=aControl;
  if (Control <> nil) And (Control.Parent <> nil) Then
    Parent:=Control.Parent;
  SetPosition;
End;

Procedure TGrabHandle.SetKind(aKind : THandleKind);
Begin
  FKind:=aKind;
  Case Kind of
    hkNW, hkSE : Cursor:=crSizeNWSE;
    hkN, hkS   : Cursor:=crSizeNS;
    hkNE, hkSW : Cursor:=crSizeNESW;
    hkE, hkW   : Cursor:=crSizeWE;
  End;
End;

Procedure TGrabHandle.SetPosition;
var L, T : Integer;
Begin
  if csDestroying in MyDesigner.ComponentState Then Exit;
  if Control=nil Then Exit;
  L:=0;
  T:=0;
  Case Kind of
    hkNW : Begin
      L:=Control.Left-2;
      T:=Control.Top-2;
    End;
    hkN  : Begin
      L:=Control.Left+((Control.Width-HandleWidth) div 2);
      T:=Control.Top-2;
    End;
    hkNE : Begin
      L:=Control.Left+Control.Width-HandleWidth+2;
      T:=Control.Top-2;
    End;
    hkE  : Begin
      L:=Control.Left+Control.Width-HandleWidth+2;
      T:=Control.Top+((Control.Height-HandleWidth) div 2);
    End;
    hkSE : Begin
      L:=Control.Left+Control.Width-HandleWidth+2;
      T:=Control.Top+Control.Height-HandleWidth+2;
    End;
    hkS  : Begin
      L:=Control.Left+((Control.Width-HandleWidth) div 2);
      T:=Control.Top+Control.Height-HandleWidth+2;
    End;
    hkSW : Begin
      L:=Control.Left-2;
      T:=Control.Top+Control.Height-HandleWidth+2;
    End;
    hkW : Begin
      L:=Control.Left-2;
      T:=Control.Top+((Control.Height-HandleWidth) div 2);
    End;
  End;
  Left:=L;
  Top:=T;
  Repaint;
End;

{ TRTDesigner ---------------------------------------------------------------------}
Constructor TRTDesigner.Create(aOwner : TComponent);

  Procedure CreateDesignerPopupMenu;
  Begin
    FDesignPop:=TPopupMenu.Create(Self);
    FDesignPop.Items.Add(NewItem(MIAlignToGrid, 0, False, True, PopAlignToGrid, 0, 'PopAlignToGrid'));
    FDesignPop.Items.Add(NewItem(MIBringToFront, 0, False, True, PopBringToFront, 0, 'PopBringToFront'));
    FDesignPop.Items.Add(NewItem(MISendToBack, 0, False, True, PopSendToBack, 0, 'PopSendToBack'));
  End;


  function NotAlone : Boolean;
  var i : Integer;
  begin
    Result:=False;
    if MyForm=nil then Exit;
    for i:=0 To MyForm.ComponentCount-1 do
      if (MyForm.Components[i] is TRTDesigner) And (MyForm.Components[i]<>Self) Then Begin
        Result:=True;
        MyForm:=nil;
        Break;
      end;
  end;

Begin
  inherited Create(aOwner);
  FDesignerState:=[dsCreating];
  MyForm:=TForm(aOwner);
  _handlescreated:=False;
  if NotAlone then begin
    if (csDesigning in ComponentState) then
      raise Exception.Create('There should only be one RTDesigner on a form!');
  end;
  FMouseStatus:=msNone;
  FControls:=TList.Create;
  FGridX:=8;
  FGridY:=8;
  if not (csDesigning in ComponentState) Then DesignServer.AddDesigner(Self);
  FAcceptedControls:=TList.Create;
  Options:=[rtdProportionalSizing, rtdSnapToGrid];
  BaseControl:=nil;
  FKeySupport:=TKeySupportObject.Create;
  FUndoList:=TList.Create;
  FUndoLimit:=255;
  FShowGrabHandles:=True;
  FShowMultiSelection:=True;
  FDesignCursor:=crDefault;
  CreateDesignerPopupMenu;
  Exclude(FDesignerState, dsCreating);
End;

Destructor TRTDesigner.Destroy;
Begin
  Include(FDesignerState, dsDestroying);
  FDesignPop.Free;
  if EditMode Then EditMode:=False;
  inherited Destroy;
  if MyForm=nil Then Exit;
  ResetUndo;
  FKeySupport.Free;
  FUndoList.Free;
  FControls.Free;
  FControls:=nil;
  FAcceptedControls.Free;
  if not (csDesigning in ComponentState) Then DesignServer.RemoveDesigner(Self);
  Exclude(FDesignerState, dsDestroying);
End;

Procedure TRTDesigner.ActivateApp;
Begin
  ShowSelection(ControlCount = 1);
  PaintSelection;
End;

Procedure TRTDesigner.AddToSelection(aControl : TControl);
var AllowSelect : Boolean;
Begin
  if Controls.IndexOf(aControl)>=0 Then Exit; { Is control already selected? }
  if aControl=BaseControl Then Exit; { Are you trying to add the BaseControl to the selection? }
  if not ControlOnBaseControl(aControl) Then Exit; { Is the control located on the BaseControl? }
  if (FAcceptedControls.Count>0) And (FAcceptedControls.IndexOf(aControl)=-1) Then Exit; { Is the control in
    the list of accepted controls? }
  AllowSelect:=True;
  if Assigned(FOnSelectControl) Then FOnSelectControl(Self, aControl, AllowSelect);
  if not AllowSelect Then Exit;
  if ControlCount=0 THen Begin
    SelectNewControl(aControl);
  End
  Else if ControlCount=1 Then Begin
    ShowSelection(False);
  End;
  Controls.Add(aControl);
  if ControlCount >= 2 Then
    PaintSelection;
//tmp
  SelectionChanged;
End;

Function TRTDesigner.AddControl(aClass : TControlClass;aParent : TWinControl;X, Y : Integer;
  aName : String;aVisible : Boolean) : TControl;
var UniqueName : String;

  Procedure GetUniqueName;
  var i : Integer;
  Begin
    i:=1;
    UniqueName:=Copy(Result.ClassName, 2, Length(Result.ClassName));
    Repeat
      if AssumedOwner.FindComponent(UniqueName+IntToStr(i))=nil Then Begin
        UniqueName:=UniqueName+IntToStr(i);
        Break;
      End;
      inc(i);
    Until False;
  End;

Begin
  Result:=aClass.Create(AssumedOwner);
  if aName='' Then Begin
    GetUniqueName;
    Result.Name:=UniqueName
  End
  Else Result.Name:=aName;
  if X<>-1 Then Result.Left:=X;
  if Y<>-1 Then Result.Top:=Y;
  Result.Parent:=aParent;
  Result.Visible:=aVisible;
End;

Procedure TRTDesigner.AddUndo(Kind : TUndoKind);
var i : Integer;
    UndoObj : TUndoObject;
Begin
  if FUndoList.Count>=UndoLimit-1 Then While FUndoList.Count>UndoLimit Do Begin
    TUndoObject(FUndoList[0]).Free;
    FUndoList.Delete(0);
  End;
  i:=FUndoList.Add(TUndoObject.Create(Kind));
  UndoObj:=TUndoObject(FUndoList[i]);
  With TWriter.Create(UndoObj.Stream, 4096) Do Begin
    For i:=0 To ControlCount-1 Do Begin
      UndoObj.Components.Add(Control[i].Name);
      WriteString(Control[i].Parent.Name);
      WriteString(Control[i].ClassName);
      WriteRootComponent(Controls[i]);
    End;
    WriteListEnd;
    Free;
  End;
  UndoObj.Stream.Position:=0;
  if Assigned(FOnUndoChange) Then FOnUndoChange(Self);
End;

Procedure TRTDesigner.AlignToGrid;
var i, x, y : Integer;
    theC : TControl;
Begin
  AddUndo(ukMove);
  For i:=0 To ControlCount-1 Do Begin
    theC:=Control[i];
    x:=theC.Left;
    y:=theC.Top;
    x:=Round(x/GridX)*GridX;
    y:=Round(y/GridY)*GridY;
    theC.Left:=x;
    theC.Top:=y;
  End;
  if Assigned(FOnMoved) Then FOnMoved(Self);
  RefreshHandlesPos;
  SendMessage(BaseControl.Handle, WM_PAINT, 0, 0);
  PaintSelection;
End;

Procedure TRTDesigner.ApplicationMessages(var Msg: TMsg; var Handled: Boolean);
var P : TPoint;
    Shift : TShiftState;


  Procedure DoMouseDown;
  var i : Integer;
      pMouse : TPoint;
      ClearSelection, DrawSelectionFrame : Boolean;
      ControlInserted : Boolean;
  Begin
    FLast:=Point(0, 0);
    FDown:=P;
    pMouse:=FMouseControl.ScreenToClient(P);
    if IsGrabHandle(FMouseControl) and (TGrabHandle(FMouseControl).MyDesigner = Self) then begin
      FDesigning:=True;
      FGrabbedHandle:=TGrabHandle(FMouseControl).Kind;
      FMouseStatus:=msSizing;
      if ControlCount=1 Then ShowSelection(False)
      Else if ControlCount>=2 Then Begin
        For i:=0 To ControlCount-1 Do RepaintControl(Controls[i], False);
      End;
      DrawSizingFrame;
    End
    Else if FMouseControl = BaseControl Then Begin
      FDesigning:=True;
      ControlInserted:=DoInsertEvent(FMouseControl, pMouse.X, pMouse.Y);
      if Assigned(FOnMouseDown) Then
        FOnMouseDown(Self, FMouseControl, mbLeft, Shift, pMouse.X, pMouse.Y);
      if not ControlInserted Then Begin
        ClearSelection:=True;
        DrawSelectionFrame:=True;
        if Assigned(FOnBaseClick) Then With BaseControl.ScreenToClient(P) Do
          FOnBaseClick(Self, ClearSelection, DrawSelectionFrame, Shift, X, Y);
        if ClearSelection Then SelectNone;
        if DrawSelectionFrame Then FMouseStatus:=msFraming;
      End;
    End
    Else Begin
      FDesigning:=True;
      if Assigned(FOnCheckIfDesigning) then
        FOnCheckIfDesigning(Self, FMouseControl, pMouse.X, pMouse.Y, FDesigning);
      if not FDesigning then begin
        SelectNone;
        Exit;
      end;
      BaseControlParentForm.ActiveControl:=nil; //Enable keyboard support
      if (FAcceptedControls.Count > 0) And (FAcceptedControls.IndexOf(FMouseControl)=-1) Then Exit;
//tmp      if not ControlOnBaseControl(FMouseControl) Then Exit;
      ControlInserted:=DoInsertEvent(FMouseControl, pMouse.X, pMouse.Y);
      if Assigned(FOnMouseDown) Then
        FOnMouseDown(Self, FMouseControl, mbLeft, Shift, pMouse.X, pMouse.Y);
      Handled:=True;
      if ControlInserted Then Begin
        FMouseStatus:=msNone;
        ShowSelection(True);
      End
      Else Begin
        ShowSelection(False);
        FMouseStatus:=msMoving;
        if (ssShift in Shift) Then Begin
          if Controls.IndexOf(FMouseControl)=-1 Then AddToSelection(FMouseControl)
          Else if ControlCount>=2 Then RemoveFromSelection(FMouseControl);
          FMouseStatus:=msNone;
        End
        Else Begin
          RemoveUnnecessaryControlsFromSelection;
          ShowSelection(False);
          if Controls.IndexOf(FMouseControl)=-1 Then Begin
            SelectNone;
            AddToSelection(FMouseControl);
          End
          Else if (ControlCount=1) And (Controls[0]=FMouseControl) Then
            SelectNewControl(FMouseControl);
          if not (ssShift in Shift) Then DrawMovingFrames;
        End;
      End;
    End;
    Case MouseStatus of
      msMoving, msSizing : Begin
        if ControlCount>0 Then LimitCursorTo(Controls[0]);
      End;
      msFraming : LimitCursorTo(BaseControl);
    End;
    SetFocus(BaseControl.Handle);
  End;

  Procedure DoDblClick(Button : TMouseButton);
  Begin
    if not ControlOnBaseControl(FMouseControl) Then Exit;
    Handled:=True;
    if Assigned(FOnControlDblClick) Then FOnControlDblClick(Self, FMouseControl);
  End;

  Procedure DoKeyDown;
  var Shift : TShiftState;
      aCtrl : TControl;
      Key : Word;
  Begin
    Shift:=KeyDataToShiftState(Msg.lParam);
    Key:=Msg.wParam;
    if Assigned(FOnKeyDown) Then FOnKeyDown(Self, Key, Shift);
    Case Key of
      VK_DELETE : Begin
        Case FKeySupport.Del of
          ksoNotActive : Handled:=False;
          ksoActive    : Handled:=True;
          ksoCtrl      : Handled:=ssCtrl in Shift;
          ksoAlt       : Handled:=ssAlt in Shift;
          ksoCtrlAlt   : Handled:=(ssCtrl in Shift) And (ssAlt in Shift);
        End;
        if Handled Then begin
          DeleteSelection;
          if Assigned(FOnDeletedControl) then
            FOnDeletedControl(Self);
        end;
      End;
      VK_INSERT : Begin
        Case FKeySupport.Ins of
          ksoNotActive : Handled:=False;
          ksoActive    : Handled:=True;
          ksoCtrl      : Handled:=ssCtrl in Shift;
          ksoAlt       : Handled:=ssAlt in Shift;
          ksoCtrlAlt   : Handled:=(ssCtrl in Shift) And (ssAlt in Shift);
        End;
        if Handled Then Begin
          if (ControlCount=0) or (ControlCount>=2) Then DoInsertEvent(BaseControl, 0, 0)
          Else DoInsertEvent(Controls[0], 0, 0);
        End;
      End;
      VK_ESCAPE : if ControlCount=1 Then Begin
        aCtrl:=Control[0];
        if aCtrl.Parent is TForm Then Begin
          SelectNone;
          Exit;
        End;
        AddToSelection(aCtrl.Parent);
        RemoveFromSelection(aCtrl);
      End;
      ord('A'), ord('a') : Begin
        Case FKeySupport.A of
          ksoNotActive : Handled:=False;
          ksoActive    : Handled:=True;
          ksoCtrl      : Handled:=ssCtrl in Shift;
          ksoAlt       : Handled:=ssAlt in Shift;
          ksoCtrlAlt   : Handled:=(ssCtrl in Shift) And (ssAlt in Shift);
        End;
        if Handled Then SelectAll;
      End;
      ord('C'), ord('c') : Begin
        Case FKeySupport.C of
          ksoNotActive : Handled:=False;
          ksoActive    : Handled:=True;
          ksoCtrl      : Handled:=ssCtrl in Shift;
          ksoAlt       : Handled:=ssAlt in Shift;
          ksoCtrlAlt   : Handled:=(ssCtrl in Shift) And (ssAlt in Shift);
        End;
        if Handled Then CopyToClipboard;
      End;
      ord('V'), ord('v') : Begin
        Case FKeySupport.V of
          ksoNotActive : Handled:=False;
          ksoActive    : Handled:=True;
          ksoCtrl      : Handled:=ssCtrl in Shift;
          ksoAlt       : Handled:=ssAlt in Shift;
          ksoCtrlAlt   : Handled:=(ssCtrl in Shift) And (ssAlt in Shift);
        End;
        if Handled Then PasteFromClipboard;
      End;
      ord('X'), ord('x') : Begin
        Case FKeySupport.X of
          ksoNotActive : Handled:=False;
          ksoActive    : Handled:=True;
          ksoCtrl      : Handled:=ssCtrl in Shift;
          ksoAlt       : Handled:=ssAlt in Shift;
          ksoCtrlAlt   : Handled:=(ssCtrl in Shift) And (ssAlt in Shift);
        End;
        if Handled Then Begin
          CopyToClipboard;
          DeleteSelection;
        End;
      End;
      ord('Z'), ord('z') : Begin
        Case FKeySupport.Z of
          ksoNotActive : Handled:=False;
          ksoActive    : Handled:=True;
          ksoCtrl      : Handled:=ssCtrl in Shift;
          ksoAlt       : Handled:=ssAlt in Shift;
          ksoCtrlAlt   : Handled:=(ssCtrl in Shift) And (ssAlt in Shift);
        End;
        if Handled Then Undo;
      End;
      ord('N'), ord('n') : Begin
        Case FKeySupport.N of
          ksoNotActive : Handled:=False;
          ksoActive    : Handled:=True;
          ksoCtrl      : Handled:=ssCtrl in Shift;
          ksoAlt       : Handled:=ssAlt in Shift;
          ksoCtrlAlt   : Handled:=(ssCtrl in Shift) And (ssAlt in Shift);
        End;
        if Handled Then SelectNone;
      End;
      { Here starts Marcondes's Ctrl + Arrows Implementation }
      VK_LEFT : Begin
        if (ssCtrl in Shift) Then MoveLeft
        Else if (ssShift in Shift) Then ResizeLeft;
      End;
      VK_UP : Begin
        if (ssCtrl in Shift) Then MoveUp
        Else if (ssShift in Shift) Then ResizeUp;
      End;
      VK_RIGHT : Begin
        if (ssCtrl in Shift) Then MoveRight
        Else if (ssShift in Shift) Then ResizeRight;
      End;
      VK_DOWN : Begin
        if (ssCtrl in Shift) Then MoveDown
        Else if (ssShift in Shift) Then ResizeDown;
      End;
    End;
    Msg.wParam:=Key;
    if ControlOnBaseControl(BaseControlParentForm.ActiveControl) Then
      Handled:=True;
  End;

  Procedure DoKeyUp;
  var Key : Word;
  Begin
    Shift:=KeyDataToShiftState(Msg.lParam);
    Key:=Msg.wParam;
    if Assigned(FOnKeyDown) Then FOnKeyDown(Self, Key, Shift);
    Msg.wParam:=Key;
  End;


  Procedure DoMouseMove;
  var pMouse : TPoint;
      IsDesigning : Boolean;
  Begin
    case MouseStatus of
      msMoving : DrawMovingFrames;
      msSizing : DrawSizingFrame;
      msFraming : DrawSelectFrame;
    end;
    FLast.X:=P.X - FDown.X;
    FLast.Y:=P.Y - FDown.Y;
    pMouse:=FMouseControl.ScreenToClient(P);
    if (rtdSnapToGrid in Options) And (MouseStatus<>msFraming) Then Begin
      FLast.X:=Round(FLast.X/GridX)*GridX;
      FLast.Y:=Round(FLast.Y/GridY)*GridY;
    end;
    case MouseStatus of
      msNone : begin
//        FDesigning:=True;
        IsDesigning:=True;
        if (FMouseControl <> BaseControl) and Assigned(FOnCheckIfDesigning) then
          FOnCheckIfDesigning(Self, FMouseControl, pMouse.X, pMouse.Y, IsDesigning);
        if not IsDesigning then
          Exit;
        BaseControlParentForm.ActiveControl:=nil;
//        if not IsGrabHandle(FMouseControl) then
//          SetCursor(Screen.Cursors[DesignCursor]);
      end;
      msMoving : begin
        DrawMovingFrames;
//        if not IsGrabHandle(FMouseControl) then
//          SetCursor(Screen.Cursors[DesignCursor]);
      end;
      msSizing : DrawSizingFrame;
      msFraming : begin
        DrawSelectFrame;
//        if not IsGrabHandle(FMouseControl) then
//          SetCursor(Screen.Cursors[DesignCursor]);
      end;
    End;
    if Assigned(FOnMouseMove) Then With FMouseControl.ScreenToClient(P) Do
      FOnMouseMove(Self, FMouseControl, Shift, X, Y, BaseControl.ScreenToClient(P));
//    FDesigning:=True;
  End;

  Procedure DoMouseUp;
  var ApplySize : Boolean;
      aCtrl : TControl;

    Procedure PositionControls;
    var i : Integer;
        theC : TControl;
    Begin
      For i:=0 To ControlCount-1 Do Begin
        theC:=Control[i];
        theC.Left:=theC.Left+FLast.X;
        theC.Top:=theC.Top+FLast.Y;
        RepaintControl(theC, False);
      End;
      if ControlCount=1 Then Begin
        RefreshHandlesPos;
        ShowSelection(True);
      End;
      PaintSelection;
    End;

    Procedure ResizeControl;
    var theC : TControl;
    Begin
      theC:=Control[0];
      if not EqualRect(theC.BoundsRect, FSizingRect) Then AddUndo(ukSize);
      With FSizingRect Do theC.SetBounds(Left, Top, Right-Left, Bottom-Top);
      RepaintControl(theC, True);
      RefreshHandlesPos;
      if Assigned(FAfterSized) Then FAfterSized(Self);
    End;

  begin
    if not FDesigning then begin
//      FDesigning:=True;
      Exit;
    end;
    if not (ssShift in Shift) Then Case MouseStatus of
      msMoving : Begin
        DrawMovingFrames;
        if (FLast.X<>0) or (FLast.Y<>0) Then AddUndo(ukMove);
        PositionControls;
        if ((FLast.X<>0) or (FLast.Y<>0))  And Assigned(FOnMoved) Then FOnMoved(Self);
        RefreshHandlesPos;
        ShowSelection(ControlCount=1);
      End;
      msSizing : Begin
        DrawSizingFrame;
        ApplySize:=True;
        if Assigned(FOnApplySize) Then Begin
          aCtrl:=Control[0];
          FOnApplySize(Self, aCtrl, aCtrl.BoundsRect, FSizingRect, ApplySize);
        End;
        if ApplySize Then ResizeControl;
        ShowSelection(True);
      End;
      msFraming : Begin
        SelectControlsInRect(DrawSelectFrame, ssCtrl in Shift);
        if ControlCount=1 Then ShowSelection(True);
        PaintSelection;
      End;
    End;
    if Assigned(FOnMouseUp) Then With FMouseControl.ScreenToClient(P) Do
      FOnMouseUp(Self, FMouseControl, mbLeft, Shift, X, Y);
    FMouseStatus:=msNone;
    PaintSelection;
    LimitCursorTo(nil);
  End;

  Procedure DoPaint;
  Begin
    if BaseControl=nil Then Exit;
    PaintSelection;
  End;

  Procedure DoRightDown;
  var aPopupMenu : TPopupMenu;
  Begin
    Case MouseStatus of
      msMoving : DrawMovingFrames;
      msSizing : DrawSizingFrame;
      msFraming : DrawSelectFrame;
    End;
    FMouseStatus:=msNone;
    if IsGrabHandle(FMouseControl) Then Begin
      SelectNone;
      Exit;
    End
    Else if FMouseControl=BaseControl Then SelectNone
    Else if not Selected(FMouseControl) Then Begin
      SelectNone;
      AddToSelection(FMouseControl);
      ShowSelection(True);
    End;
    if Assigned(FOnGetPopupMenu) then begin
      aPopupMenu:=nil;
      FOnGetPopupMenu(Self, aPopupMenu);
      if aPopupMenu<>nil then
        aPopupMenu.Popup(P.X, P.Y);
    end
    else begin
      PopulateDesignPopup;
      FDesignPop.Popup(P.X, P.Y);
    end;
    Handled:=True;
  End;

  Procedure DoRightUp;
  Begin
    if Assigned(FOnMouseDown) Then With FMouseControl.ScreenToClient(P) Do
      FOnMouseDown(Self, FMouseControl, mbRight, Shift, X, Y);
  End;

Begin
  Handled:=False;
  if (csDesigning in ComponentState) or not EditMode Then Exit;
  GetCursorPos(P);
  FMouseControl:=GetControlUnderMouse;
  if FMouseControl = nil Then Exit;
  if (FMouseControl is TForm) And not (FMouseControl = BaseControl) And
    (MouseStatus = msNone) Then Exit;
  if (FMouseControl <> BaseControl) And not (FMouseControl is TGrabHandle) And
    (MouseStatus = msNone) Then Begin
    While FMouseControl.Owner <> AssumedOwner Do Begin { Make sure the control that's pointed at has the correct owner so that
     it isn't a button on a TDBNavigator or likewise.}
      FMouseControl:=FMouseControl.Parent;
      if FMouseControl = nil Then Exit;
    End;
  End;

  if (FMouseControl <> BaseControl) and not (FMouseControl is TGrabHandle) and
     (not ControlOnBaseControl(FMouseControl) and
     (MouseStatus = msNone)) then
    Exit;

  Shift:=KeysToShiftState(Msg.wParam);
  Case Msg.Message of
    WM_LBUTTONDOWN{, WM_NCLBUTTONDOWN}   : DoMouseDown;
    WM_LBUTTONDBLCLK : DoDblClick(mbLeft);
    WM_LBUTTONUP, WM_NCLBUTTONUP     : DoMouseUp;
    WM_MOUSEMOVE, WM_NCMOUSEMOVE     : DoMouseMove;
    WM_RBUTTONDOWN   : DoRightDown;
    WM_RBUTTONDBLCLK : DoDblClick(mbRight);
    WM_RBUTTONUP     : DoRightUp;
    WM_PAINT : DoPaint;
    WM_KEYDOWN : begin
//      if BaseControlParentForm.ActiveControl = nil then
//      if (Msg.hwnd = BaseControlParentForm.Handle) then
      if FDesigning then
        DoKeyDown;
    end;
    WM_KEYUP : begin
      if BaseControlParentForm.ActiveControl = nil then
      DoKeyUp;
    end;
  End;
End;

procedure TRTDesigner.ReadComponentsProc(Component: TComponent);
begin
  if dsLoading in FDesignerState then
    Exit;
  if Component is TControl then
    AddToSelection(TControl(Component));
end;

Procedure TRTDesigner.RefreshHandlesPos;
var i : Integer;
Begin
  if ControlCount>=2 then PaintSelection;
  if not _handlescreated Then Exit;
  For i:=0 To 7 Do FHandles[i].SetPosition;
End;

Procedure TRTDesigner.ReaderSetName(Reader: TReader; Component: TComponent; var Name: string);
var n : Integer;
    aPos : TPoint;

  Procedure CheckName;
  var aComp : TComponent;
  Begin
    aComp:=AssumedOwner.FindComponent(Name);
    if (aComp=nil) or (aComp=Component) Then Exit;
    inc(n);
    Name:=Copy(Component.ClassName, 2, Length(Component.ClassName))+IntToStr(n);
    CheckName;
  End;

Begin
  n:=0;
  CheckName;
  if Component.InheritsFrom(TControl) Then With Component As TControl Do Begin
    aPos:=Point(Left, Top);
    While BaseControl.ControlAtPos(aPos, True)<>nil Do Begin
      aPos:=Point(aPos.X+GridX, aPos.Y+GridY);
    End;
    Left:=aPos.X;
    Top:=aPos.Y;
  End;
  if n>0 Then FReadErrors.Add(Component);
  if Component is TControl Then
    AddToSelection(TControl(Component));
End;

procedure TRTDesigner.RemoveOtherRTDesigners;
var i : Integer;
    aForm : TForm;
Begin
  aForm:=BaseControlParentForm;
  if aForm = nil then Exit;
  for i:=aForm.ComponentCount-1 downto 0 do
    if aForm.Components[i].InheritsFrom(TRTDesigner) and
      (aForm.Components[i] <> Self) then
      TRTDesigner(aForm.Components[i]).Free;
end;

Function TRTDesigner.GetControlCount : Integer;
Begin
  Result:=Controls.Count;
End;

Function TRTDesigner.ControlOnBaseControl(aControl : TControl) : Boolean;

  Function HasParent(aControl : TControl;aParent : TWinControl) : Boolean;
  var theParent : TWinControl;
  Begin
    Result:=False;
    theParent:=aControl.Parent;
    Repeat
      if theParent=aParent Then Result:=True;
      theParent:=theParent.Parent;
    Until Result or (theParent=nil);
  End;

Begin
  Result:=False;
  if (BaseControl=nil) or (aControl=nil) or (aControl.Parent=nil) Then Exit;
  if AssumedOwner<>aControl.Owner Then Exit; { The control must be owned by the parent form,
   otherwise it might be a button in a TDBNavigator that we are trying to select. }
  Result:=HasParent(aControl, BaseControl); { Check if the control is placed on the BaseControl
    or on a control on it... and so on }
End;

Procedure TRTDesigner.BringToFront;
var i : Integer;
Begin
  For i:=0 To ControlCount-1 Do
    Control[i].BringToFront;
End;

Procedure TRTDesigner.ClearBaseControl;
var i : Integer;
Begin
  if BaseControl=nil Then Exit;
  SelectNone;
  For i:=BaseControl.ControlCount-1 Downto 0 Do if not IsGrabHandle(BaseControl.Controls[i]) Then Begin
    if Assigned(FOnRemoveControl) Then FOnRemoveControl(Self, BaseControl.Controls[i]);
    BaseControl.Controls[i].Free;
  End;
  if Assigned(FAfterBCClear) Then FAfterBCClear(Self);
End;


Procedure TRTDesigner.CopyToClipboard;
var i : Integer;
    Stream, txtStream : TMemoryStream;
    txt : TStringList;
    Str : PChar;
Begin
  if ControlCount=0 Then Exit;
  RemoveUnnecessaryControlsFromSelection;
  PaintSelection;
  Stream:=TMemoryStream.Create;
  With TWriter.Create(Stream, 4096) Do Begin
    Root:=AssumedOwner;
    For i:=0 To ControlCount-1 Do Begin
      WriteSignature;
      WriteComponent(Controls[i]);
    End;
    WriteListEnd;
    Free;
  End;
  Stream.Position:=0;
  txtStream:=TMemoryStream.Create;
  While Stream.Position<Stream.Size-1 Do
    ObjectBinaryToText(Stream, txtStream);
  txtStream.Position:=0;
  txt:=TStringList.Create;
  txt.LoadFromStream(txtStream);
  Str:=txt.GetText;
  Clipboard.SetTextBuf(Str);
  StrDispose(Str);
  txtStream.Free;
  Stream.Free;
  txt.Free;
End;


Procedure TRTDesigner.CreateHandles;
var i : Integer;
Begin
  For i:=0 To 7 Do
  begin
    if Assigned(FHandles[i]) then
      Continue;
    FHandles[i]:=TGrabHandle.CreateNew(Self, BaseControl, nil, THandleKind(i));
  end;
  _handlescreated:=True;
End;

Function TRTDesigner.ControlAspect(aControl : TControl) : Extended;
Begin
  Result:=aControl.Width/aControl.Height;
End;

Procedure TRTDesigner.DeactivateApp;
Begin
  Case MouseStatus of
    msMoving : DrawMovingFrames;
    msSizing : DrawSizingFrame;
    msFraming : DrawSelectFrame;
  End;
  ShowSelection(ControlCount=1);
  FMouseStatus:=msNone;
  PaintSelection;
End;

Procedure TRTDesigner.DeleteSelection;
var i : Integer;
    Ctrl : TControl;
    DoDelete : Boolean;
Begin
  DoDelete:=True;
  if Assigned(FOnDelete) Then FOnDelete(Self, DoDelete);
  if not DoDelete Then Exit;
  RemoveUnnecessaryControlsFromSelection;
  AddUndo(ukDelete);
  For i:=ControlCount-1 Downto 0 Do Begin
    Ctrl:=Controls[i];
    RemoveFromSelection(Ctrl);
    if Assigned(FOnRemoveControl) Then FOnRemoveControl(Self, Ctrl);
    Ctrl.Free;
  End;
  Controls.Clear;
  FDesigning:=False;
End;

Procedure TRTDesigner.DestroyHandles;
var i : Integer;
Begin
  For i:=0 To 7 Do
    FHandles[i].Free;
  _handlescreated:=False;
End;


Procedure TRTDesigner.DrawFrame(Handle : THandle;R : TRect);
var ParentDC: HDC;
    aCanvas : TCanvas;
Begin
  ParentDC:=GetDCEx(Handle, 0, DCX_CACHE or DCX_CLIPSIBLINGS or DCX_LOCKWINDOWUPDATE);
  aCanvas:=TCanvas.Create;
  aCanvas.Handle:=ParentDC;
  With aCanvas Do Begin
    Pen.Width:=2;
    Pen.Color:=clGray;
    Pen.Style:=psInsideFrame;
    Pen.Mode:=pmXor;
    Brush.Style:=bsClear;
    With R Do
      Rectangle(Left, Top, Right, Bottom);
  End;
  ReleaseDC(BaseControlParentForm.Handle, ParentDC);
  aCanvas.Free;
End;

Procedure TRTDesigner.DrawMovingFrames;
var i : Integer;
    theC : TControl;
    r : TRect;
Begin
  For i:=0 To ControlCount-1 Do Begin
    theC:=Control[i];
    r:=theC.BoundsRect;
    OffsetRect(r, FLast.X, FLast.Y);
    DrawFrame(theC.Parent.Handle, r);
  End;
End;

Function TRTDesigner.DrawSelectFrame : TRect;
var ParentDC: HDC;
    aCanvas : TCanvas;
    X1, Y1, X2, Y2 : Integer;
Begin
  ParentDC:=GetDCEx(BaseControl.Handle, 0, DCX_CACHE or DCX_CLIPSIBLINGS or DCX_LOCKWINDOWUPDATE);
  aCanvas:=TCanvas.Create;
  aCanvas.Handle:=ParentDC;
  With aCanvas Do Begin
    Pen.Style:=psDot;
    Pen.Mode:=pmXor;
    With BaseControl.ScreenToClient(FDown) Do Begin
      X1:=X;
      Y1:=Y;
    End;
    X2:=X1+FLast.X;
    Y2:=Y1+FLast.Y;
    Result:=Rect(X1, Y1, X1, Y1);
    if Y2<Y1 Then Result.Top:=Y2
    Else Result.Bottom:=Y2;
    if X2<X1 Then Result.Left:=X2
    Else Result.Right:=X2;
    PolyLine([Point(X1,Y1),Point(X2,Y1),Point(X2,Y2)]);
    PolyLine([Point(X1,Y1),Point(X1,Y2),Point(X2,Y2)]);
  End;
  ReleaseDC(BaseControlParentForm.Handle, ParentDC);
  aCanvas.Free;
End;

Procedure TRTDesigner.DrawSizingFrame;

  Function GetSizingRect(aCtrl : TControl;pDistance : TPoint) : TRect;
  var L, T, H, W : Integer;
      dH, dW : Integer;
  Begin
    L:=aCtrl.Left;
    T:=aCtrl.Top;
    W:=aCtrl.Width;
    H:=aCtrl.Height;
    Case FGrabbedHandle of
      hkNW : Begin
        dW:=pDistance.X;
        dH:=pDistance.Y;
        if rtdProportionalSizing in Options Then Begin
          if -dW>-dH Then dH:=Round(dW/ControlAspect(Controls[0]))
          Else dW:=Round(dH*ControlAspect(Controls[0]));
        End;
        if rtdCenterSizing in Options Then Begin
          W:=W-(dW*2);
          L:=L+dW;
          H:=H-(dH*2);
          T:=T+dH;
        End
        Else Begin
          L:=L+dW;
          W:=W-dW;
          T:=T+dH;
          H:=H-(dH*1);
        End;
      End;
      hkN : Begin
        T:=T+pDistance.Y;
        if rtdCenterSizing in Options Then H:=H-(pDistance.Y*2)
        Else H:=H-pDistance.Y;
      End;
      hkNE : Begin
        dW:=pDistance.X;
        dH:=pDistance.Y;
        if rtdProportionalSizing in Options Then Begin
          if dW>-dH Then dH:=-Round(dW/ControlAspect(Controls[0]))
          Else dW:=-Round(dH*ControlAspect(Controls[0]));
        End;
        if rtdCenterSizing in Options Then Begin
          W:=W+(dW*2);
          L:=L-dW;
          H:=H-(dH*2);
          T:=T+dH;
        End
        Else Begin
          W:=W+dW;
          T:=T+dH;
          H:=H-(dH*1);
        End;
      End;
      hkE : Begin
        W:=W+pDistance.X;
        if rtdCenterSizing in Options Then Begin
          L:=L-pDistance.X;
          W:=W+pDistance.X;
        End;
      End;
      hkSE : Begin
        dW:=pDistance.X;
        dH:=pDistance.Y;
        if rtdProportionalSizing in Options Then Begin
          if dW>dH Then dH:=Round(dW/ControlAspect(Controls[0]))
          Else dW:=Round(dH*ControlAspect(Controls[0]));
        End;
        if rtdCenterSizing in Options Then Begin
          W:=W+(dW*2);
          L:=L-dW;
          H:=H+(dH*2);
          T:=T-dH;
        End
        Else Begin
          W:=W+dW;
          H:=H+dH;
        End;
      End;
      hkS : Begin
        H:=H+pDistance.Y;
        if rtdCenterSizing in Options Then Begin
          H:=H+pDistance.Y;
          T:=T-pDistance.Y;
        End;
      End;
      hkSW : Begin
        dW:=pDistance.X;
        dH:=pDistance.Y;
        if rtdProportionalSizing in Options Then Begin
          if -dW>dH Then dH:=-Round(dW/ControlAspect(Controls[0]))
          Else dW:=-Round(dH*ControlAspect(Controls[0]));
        End;
        if rtdCenterSizing in Options Then Begin
          W:=W-(dW*2);
          L:=L+dW;
          H:=H+(dH*2);
          T:=T-dH;
        End
        Else Begin
          L:=L+dW;
          W:=W-dW;
          H:=H+dH;
        End;
      End;
      hkW : Begin
        L:=L+pDistance.X;
        W:=W-pDistance.X;
        if rtdCenterSizing in Options Then W:=W-pDistance.X;
      End;
    End;
    Result:=Bounds(L, T, W, H);
  End;

Begin
  FSizingRect:=GetSizingRect(Control[0], FLast);
  DrawFrame(Control[0].Parent.Handle, FSizingRect);
End;

Function TRTDesigner.GetActive : Boolean;
Begin
  if BaseControl=nil Then Result:=False
  Else Begin
{$ifdef Ver80}
    if BaseControl is TForm Then
{$else}{$ifdef Ver90}
    if BaseControl is TForm Then
{$else}
    if BaseControl is TCustomForm Then
{$endif}{$endif}
      Result:=Screen.ActiveForm=BaseControl
    Else
{$ifdef Ver80}
    Result:=(Screen.ActiveForm=nil) or (Screen.ActiveForm=BaseControlParentForm);
{$else}
    Result:=Screen.ActiveForm=BaseControlParentForm;
{$endif}
  End;
End;

Function TRTDesigner.AssumedOwner : TWinControl;
Begin
  if BaseControl=nil Then Result:=nil
  Else if BaseControl.InheritsFrom(TForm) Then Result:=BaseControl
  Else if BaseControl.Owner.InheritsFrom(TWinControl) Then Result:=TWinControl(BaseControl.Owner)
  Else Result:=nil;
End;

Function TRTDesigner.BaseControlParentForm : TForm;
var aControl : TControl;
Begin
  Result:=nil;
  if BaseControl=nil Then Exit;
  if BaseControl.InheritsFrom(TForm) Then Result:=TForm(BaseControl)
  Else Begin
    aControl:=BaseControl.Parent;
    While (aControl<>nil) And not aControl.InheritsFrom(TForm) Do aControl:=aControl.Parent;
    if (aControl<>nil) And aControl.InheritsFrom(TForm) Then Result:=TForm(aControl)
    Else Result:=nil;
  End;
End;

Function TRTDesigner.DoInsertEvent(aParent : TControl;X, Y : Integer) : Boolean;
var aClass : TControlClass;
    Ctrl, oldClipControl : TControl;
    Parent : TWinControl;
    CtrlPos : TPoint;
Begin
  aClass:=nil;
  Result:=False;
  oldClipControl:=FClipControl;
  LimitCursorTo(nil);
  CtrlPos:=Point(X, Y);
  if not (aParent is TWinControl) Then Begin
    Parent:=aParent.Parent;
    CtrlPos:=Parent.ScreenToClient(aParent.ClientToScreen(CtrlPos));
  End
  Else Begin
    Parent:=TWinControl(aParent);
    if not (csAcceptsControls in Parent.ControlStyle) Then Begin
      Parent:=Parent.Parent;
      if Parent=nil Then Exit;
      CtrlPos:=Parent.ScreenToClient(aParent.ClientToScreen(CtrlPos));
    End;
  End;
  if Assigned(FOnInsert) Then FOnInsert(Self, Parent, aClass);
  if aClass<>nil Then Begin
    FDesigning:=True;
    Ctrl:=AddControl(aClass, Parent, CtrlPos.X, CtrlPos.Y, '', True);
    Result:=True;
    SelectNone;
    AddToSelection(Ctrl);
    ShowSelection(True);
    AddUndo(ukInsert);
    if Assigned(FAfterInsert) Then FAfterInsert(Self, Ctrl);
  End;
  LimitCursorTo(oldClipControl);
End;

Function TRTDesigner.GetControl(index : Integer) : TControl;
Begin
  if (index>=0) And (index<=ControlCount-1) Then
    Result:=TControl(Controls[index])
  Else Result:=nil;
End;

Function TRTDesigner.GetUndoKind : TUndoKind;
Begin
  if FUndoList.Count=0 Then Result:=ukNone
  Else Result:=TUndoObject(FUndoList[FUndoList.Count-1]).Kind;
End;

Function TRTDesigner.GetUndoString : String;
Begin
  Result:='Undo '+CUndoString[UndoKind];
End;

Function TRTDesigner.GetVer : String;
Begin
  Result:=RuntimeDesignerVersionStr;
End;

Function TRTDesigner.IsGrabHandle(aControl : TControl) : Boolean;
Begin
  Result:=aControl is  TGrabHandle;
End;

Procedure TRTDesigner.LimitCursorTo(aControl : TControl); { by Sergey Mishkovskiy }
var rParent, rBaseControl : TRect;
Begin
  FClipControl:=aControl;
  if aControl=nil Then Begin
    ClipCursor(nil);
    Exit;
  End;
  if aControl=BaseControl Then Begin
    rParent.TopLeft:=BaseControl.ClientToScreen(Point(0, 0));
    rParent.BottomRight:=BaseControl.ClientToScreen(Point(BaseControl.ClientWidth,
                                                               BaseControl.ClientHeight));
  End
  Else Begin
    rParent.TopLeft:=aControl.Parent.ClientToScreen(Point(0, 0));
    rParent.BottomRight:=aControl.Parent.ClientToScreen(Point(aControl.Parent.ClientWidth,
                                                              aControl.Parent.ClientHeight));
    rBaseControl.TopLeft:=BaseControl.ClientToScreen(Point(0, 0));
    rBaseControl.BottomRight:=BaseControl.ClientToScreen(Point(BaseControl.ClientWidth,
                                                               BaseControl.ClientHeight));
{$ifdef ver80}
    if IntersectRect(rBaseControl, rParent, rBaseControl)<>0 Then
{$else}
    if IntersectRect(rBaseControl, rParent, rBaseControl) Then
{$endif}
      rParent:=rBaseControl;
  End;
  ClipCursor(@rParent);
End;

Procedure TRTDesigner.LoadBCFromFile(const aFileName : String);
var Stream : TFileStream;
Begin
  Stream:=TFileStream.Create(aFileName, fmOpenRead);
  Try
    LoadBCFromStream(Stream);
  Finally
    Stream.Free;
  End;
End;

Procedure TRTDesigner.LoadBCFromStream(Stream : TStream);
Begin
  Include(FDesignerState, dsLoading);
  ClearBaseControl;
  with TReader.Create(Stream, 4096) Do Try
    ReadRootComponent(BaseControl);
    if not (BaseControl is TForm) then
    begin
      ReadComponents(AssumedOwner, BaseControl, ReadComponentsProc);
    end;
  finally
    Free;
  end;
  Exclude(FDesignerState, dsLoading);
  RemoveOtherRTDesigners;
End;


Procedure TRTDesigner.Loaded;
Begin
  inherited Loaded;
End;

procedure TRTDesigner.MoveLeft;
var i : Integer;
begin
  For i:=0 To (ControlCount-1) do
    Control[i].Left:=Control[i].Left-1;
  RefreshHandlesPos;
  if Assigned(FOnMoved) Then FOnMoved(Self);
end;

procedure TRTDesigner.MoveRight;
var i : Integer;
begin
  For i:=0 To (ControlCount-1) do
    Control[i].Left:=Control[i].Left+1;
  RefreshHandlesPos;
  if Assigned(FOnMoved) Then FOnMoved(Self);
end;

procedure TRTDesigner.MoveUp;
var i : Integer;
begin
  For i:=0 To (ControlCount-1) do
    Control[i].Top:=Control[i].Top-1;
  RefreshHandlesPos;
  if Assigned(FOnMoved) Then FOnMoved(Self);
end;

procedure TRTDesigner.MoveDown;
var i : Integer;
begin
  For i:=0 To (ControlCount-1) do
    Control[i].Top:=Control[i].Top+1;
  RefreshHandlesPos;
  if Assigned(FOnMoved) Then FOnMoved(Self);
end;

Procedure TRTDesigner.Notification(aComponent : TComponent;Operation : TOperation);
var i : Integer;
Begin
  inherited Notification(aComponent, Operation);
  if Operation=opRemove Then Begin
    if (aComponent=BaseControl) Then BaseControl:=nil;
    if Controls=nil Then Exit;
    i:=Controls.IndexOf(aComponent);
    if i>=0 Then Begin
      RemoveFromSelection(Controls[i]);
    End;
  End;
End;

Procedure TRTDesigner.PaintHandlesOnCanvas(aControl : TControl);
var DC: HDC;
    aCanvas : TCanvas;
    Handle : THandle;
    aRect : TRect;
    colHighlight, colShadow, colFace : TColor;

  Procedure DoRectangle(X1, Y1 : Integer);
  var aRect : TRect;
  Begin
    aRect:=Bounds(X1, Y1, 5, 5);
    aCanvas.Brush.Color:=colFace;
    aCanvas.FillRect(aRect);
    Frame3D(aCanvas, aRect, colHighlight, colShadow, 1);
  End;

Begin
  aRect:=aControl.ClientRect;
  if aControl is TGraphicControl Then Begin
    Handle:=aControl.Parent.Handle;
    OffsetRect(aRect, aControl.Left, aControl.Top);
  End
  Else if aControl is TWinControl Then Handle:=TWinControl(aControl).Handle
  Else Handle:=0;
  DC:=GetDCEx(Handle, 0, DCX_CACHE or DCX_CLIPSIBLINGS or DCX_LOCKWINDOWUPDATE);
  aCanvas:=TCanvas.Create;
  aCanvas.Handle:=DC;
  Case GrabHandleKind of
    gkHighContrast : Begin
      colHighlight:=clBlack;
      colShadow:=clBlack;
      colFace:=clWhite;
    End;
    gkDelphi : Begin
      colHighlight:=clGray;
      colShadow:=clGray;
      colFace:=clGray;
    End;
    gk3D : Begin
      colHighlight:=clBtnHighlight;
      colShadow:=clBtnShadow;
      colFace:=clBtnFace;
    End;
  End;
  With aRect Do Begin
    DoRectangle(Left, Top); { hkNW }
    DoRectangle(Right-HandleWidth, Top); { hkNE }
    DoRectangle(Right-HandleWidth, Bottom-HandleWidth); { hkSE }
    DoRectangle(Left, Bottom-HandleWidth); { hkSW }
  End;
  ReleaseDC(Handle, DC);
  aCanvas.Free;
End;


Procedure TRTDesigner.PaintSelection;
var i : Integer;
Begin
  if (ControlCount < 2) or not ShowMultiSelection then Exit;
  if MouseStatus<>msNone Then Exit;
  For i:=0 To ControlCount-1 Do Begin
    PaintHandlesOnCanvas(Controls[i]);
  End;
End;

Procedure TRTDesigner.PasteFromClipboard;
var Stream, txtStream : TMemoryStream;
    txt : TStringList;
    ptxt : PChar;
    aParent : TWinControl;
    Reader : TReader;
    i : Integer;
Begin
  if (ControlCount=1) And (Control[0] is TWinControl) And
     (csAcceptsControls in Control[0].ControlStyle) Then
       aParent:=TWinControl(Control[0])
  Else aParent:=BaseControl;
  txtStream:=TMemoryStream.Create;
  txt:=TStringList.Create;
  ptxt:=StrAlloc(ClipboardTextBufSize);
  Clipboard.GetTextBuf(ptxt, ClipboardTextBufSize);
  txt.SetText(ptxt);
  StrDispose(ptxt);
  txt.SaveToStream(txtStream);
  txt.Free;  
  txtStream.Position:=0;
  Stream:=TMemoryStream.Create;
  While txtStream.Position<txtStream.Size-1 Do
    ObjectTextToBinary(txtStream, Stream);
  With TWriter.Create(Stream, 4096) Do Begin
    WriteListEnd;
    Free;
  End;
  Stream.Position:=0;
  SelectNone;
  FReadErrors:=TList.Create;
  Reader:=TReader.Create(Stream, 4096);
  Reader.OnSetName:=ReaderSetName;
  Reader.ReadComponents(AssumedOwner, aParent, ReadComponentsProc);
  Reader.Free;
  Stream.Free;
  txtStream.Free;
  For i:=0 To FReadErrors.Count-1 Do if TObject(FReadErrors[i]).InheritsFrom(TControl) Then
    With TControl(FReadErrors[i]) Do Begin
    Left:=Left+GridX;
    Top:=Top+GridY;
  End;
  FReadErrors.Free;
  RefreshHandlesPos;
  ShowSelection(ControlCount=1);
  AddUndo(ukInsert);
  FDesigning:=True;
End;

Procedure TRTDesigner.PopAlignToGrid(Sender : TObject);
Begin
  AlignToGrid;
End;

Procedure TRTDesigner.PopBringToFront(Sender : TObject);
Begin
  BringToFront;
End;

Procedure TRTDesigner.PopSendToBack(Sender : TObject);
Begin
  SendToBack;
End;


Procedure TRTDesigner.PopulateDesignPopup;
Begin
  While FDesignPop.Items.Count>0 Do
    FDesignPop.Items.Delete(FDesignPop.Items.Count-1);
  if ControlCount>=1 Then Begin
    FDesignPop.Items.Add(NewItem(MIAlignToGrid, 0, False, True, PopAlignToGrid, 0, 'PopAlignToGrid'));
    FDesignPop.Items.Add(NewItem(MIBringToFront, 0, False, True, PopBringToFront, 0, 'PopBringToFront'));
    FDesignPop.Items.Add(NewItem(MISendToBack, 0, False, True, PopSendToBack, 0, 'PopSendToBack'));
  End;
  if Assigned(FOnPopulateMenu) Then FOnPopulateMenu(Self, FDesignPop);
End;

Procedure TRTDesigner.RemoveUnnecessaryControlsFromSelection;
var i : Integer;
Begin
  For i:=ControlCount-1 Downto 0 Do if Controls.IndexOf(Control[i].Parent)>=0 Then
    RemoveFromSelection(Controls[i]);
End;


Procedure TRTDesigner.RepaintControl(aCtrl : TControl;Inflated : Boolean);
var r : TRect;

  Procedure DoInvalidate(aHandle : THandleKind);
  var hR : TRect;
  Begin
    hR:=GetHandleRect(r, aHandle);
    InvalidateRect(aCtrl.Parent.Handle, @hR, True);
  End;

Begin
  if aCtrl is TGraphicControl Then Begin
    r:=aCtrl.BoundsRect;
    DoInvalidate(hkNW);
    DoInvalidate(hkNE);
    DoInvalidate(hkSE);
    DoInvalidate(hkSW);
    if Inflated Then Begin
      DoInvalidate(hkN);
      DoInvalidate(hkE);
      DoInvalidate(hkS);
      DoInvalidate(hkW);
    End;
    SendMessage(aCtrl.Parent.Handle, WM_PAINT, 0, 0);
  End
  Else Begin
    aCtrl.Repaint;
    if not Inflated Then Exit;
    r:=aCtrl.BoundsRect;
    InvalidateRect(aCtrl.Parent.Handle, @r, True);
    SendMessage(aCtrl.Parent.Handle, WM_PAINT, 0, 0);
  End;
End;

Procedure TRTDesigner.RemoveFromSelection(aControl : TControl);
var i : Integer;
Begin
  if not (csDestroying in ComponentState) Then For i:=0 To ControlCount-1 Do
    RepaintControl(Controls[i], False);
  i:=Controls.IndexOf(aControl);
  if i>=0 Then Controls.Delete(i);
  if ControlCount=1 Then Begin
    SelectNewControl(Controls[0]);
    ShowSelection(True);
  End
  Else if ControlCount=0 Then Begin
    SelectNewControl(nil);
    ShowSelection(False);
  End;
  SelectionChanged;
End;

Procedure TRTDesigner.ResetUndo;
var i : Integer;
Begin
  For i:=0 To FUndoList.Count-1 Do
    TMemoryStream(FUndoList[i]).Free;
  FUndoList.Clear;
End;

procedure TRTDesigner.ResizeLeft;
var i : Integer;
begin
  For i:=0 To (ControlCount-1) do Begin
    Control[i].Width:=Control[i].Width-1;
    Control[i].Refresh;
  End;
  RefreshHandlesPos;
  if Assigned(FAfterSized) Then FAfterSized(Self);
end;

procedure TRTDesigner.ResizeRight;
var i : Integer;
begin
  For i:=0 To (ControlCount-1) do Begin
    Control[i].Width:=Control[i].Width+1;
    Control[i].Refresh;
  End;
  RefreshHandlesPos;
  if Assigned(FAfterSized) Then FAfterSized(Self);
end;

procedure TRTDesigner.ResizeUp;
var i : Integer;
begin
  For i:=0 To (ControlCount-1) do Begin
    Control[i].Height:=Control[i].Height-1;
    Control[i].Refresh;
  End;
  RefreshHandlesPos;
  if Assigned(FAfterSized) Then FAfterSized(Self);
end;

procedure TRTDesigner.ResizeDown;
var i : Integer;
begin
  For i:=0 To (ControlCount-1) do Begin
    Control[i].Height:=Control[i].Height+1;
    Control[i].Refresh;
  End;
  RefreshHandlesPos;
  if Assigned(FAfterSized) Then FAfterSized(Self);
end;

Procedure TRTDesigner.SaveBCToFile(const aFileName : String);
var Stream : TFileStream;
Begin
  Stream:=TFileStream.Create(aFileName, fmCreate or fmOpenWrite);
  Try
    SaveBCToStream(Stream);
  Finally
    Stream.Free;
  End;
End;

Procedure TRTDesigner.SaveBCToStream(Stream : TStream);
var i : Integer;
    oldName : String;
Begin
  With TWriter.Create(Stream, 4096) Do Try
    oldName:=Self.Name;
    Self.Name:=oldName+'tmp';
    WriteRootComponent(BaseControl);
    if not (BaseControl is TForm) Then Begin
      Root:=AssumedOwner;
      For i:=0 To BaseControl.ControlCount-1 Do Begin
        if IsGrabHandle(BaseControl.Controls[i]) Then Continue;
        if BaseControl.Controls[i].InheritsFrom(TRTDesigner) then Continue;
        WriteSignature;
        WriteComponent(BaseControl.Controls[i]);
      End;
      WriteListEnd;
    End;
  Finally
    Free;
    Self.Name:=oldName;
  End;
End;


Procedure TRTDesigner.SelectControlsInRect(aRect : TRect;AllControls : Boolean);
var theR : TRect;

  Function IsSelected(cRect : TRect) : Boolean;
  var l, t, r, b : Boolean;
  Begin
    l:=(theR.Right>cRect.Left) And (theR.Left<cRect.Right);
    r:=(theR.Left<cRect.Right) And (theR.Right>cRect.Left);
    t:=(theR.Bottom>cRect.Top) And (theR.Top<cRect.Bottom);
    b:=(theR.Top<cRect.Bottom) And (theR.Bottom>cRect.Top);
    Result:=(l or r) And (t or b);
  End;

  Procedure DoSelect(aControl : TWinControl);
  var i : Integer;
      theC : TControl;
  Begin
    For i:=0 To aControl.ControlCount-1 Do Begin
      theC:=aControl.Controls[i];
      if not IsGrabHandle(theC) And IsSelected(ClientRectToScreen(theC)) Then
        AddToSelection(theC);
      if AllControls And (theC is TWinControl) Then
        DoSelect(theC As TWinControl);
    End;
  End;

Begin
  theR:=aRect;
  With BaseControl.ClientToScreen(Point(0, 0)) Do
    OffsetRect(theR, X, Y);
  DoSelect(BaseControl);
  SelectionChanged;
End;

Procedure TRTDesigner.SelectAll;

  Procedure DoSelect(aControl : TWinControl);
  var i : Integer;
      theC : TControl;
  Begin
    For i:=0 To aControl.ControlCount-1 Do Begin
      theC:=aControl.Controls[i];
      if theC is TGrabHandle Then Continue;
      AddToSelection(theC);
      if theC is TWinControl Then DoSelect(TWinControl(theC));
    End;
  End;

Begin
  DoSelect(BaseControl);
  SelectionChanged;
End;

Function TRTDesigner.Selected(aControl : TControl): Boolean;
Begin
  Result:=FControls.IndexOf(aControl)>=0;
End;

procedure TRTDesigner.SelectionChanged;
begin
  if Assigned(FOnSelectionChanged) then
    FOnSelectionChanged(Self);
  if (Controls.Count > 0) and ControlOnBaseControl(BaseControlParentForm.ActiveControl) then
    BaseControlParentForm.ActiveControl:=nil;
end;

Procedure TRTDesigner.SelectNewControl(aControl : TControl);
var i : Integer;
Begin
  if csDestroying in ComponentState Then Exit;
  if not _handlescreated Then CreateHandles;
  For i:=0 To 7 Do Begin
    FHandles[i].Control:=aControl;
    FHandles[i].BringToFront;
  End;
End;

Procedure TRTDesigner.SelectNone;
var i : Integer;
Begin
  if ControlCount>=2 Then For i:=0 To ControlCount-1 Do
    RepaintControl(Controls[i], False);
  Controls.Clear;
  ShowSelection(False);
  SelectionChanged;
End;

Procedure TRTDesigner.SendToBack;
var i : Integer;
Begin
  For i:=ControlCount-1 Downto 0 Do
    Control[i].SendToBack;
End;

Procedure TRTDesigner.SetBaseControl(aControl : TWinControl);
Begin
  if csDestroying in ComponentState Then Begin
    FBaseControl:=nil;
    Exit;
  End;
  if aControl<>BaseControl Then Begin
    ResetUndo;
  End;
  FBaseControl:=aControl;
  if FBaseControl<>nil Then Begin
    SendMessage(BaseControl.Handle, WM_PAINT, 0, 0);
  End;
  if not _handlescreated Then Exit;
  DestroyHandles;
  CreateHandles;
End;

Procedure TRTDesigner.SetEditMode(Value : Boolean);
Begin
  if csDestroying in ComponentState then
    Exit;
  FEditMode:=Value;
  if csDesigning in ComponentState then
    Exit;
  if not EditMode then
    SelectNone;
  if BaseControl <> nil then
    BaseControl.Invalidate;
  if Assigned(FOnEditModeChange) then
    FOnEditModeChange(Self);
  Application.OnMessage:=DesignServer.ApplicationMessage; // === >  Add by Asi
End;

Procedure TRTDesigner.SetVer(const A : String);
Begin
End;


Procedure TRTDesigner.ShowSelection(Show : Boolean);
var i : Integer;
Begin
  if csDestroying in ComponentState Then Exit;
  if not _handlescreated Then Exit;
  if (ControlCount >= 2) Then Begin
    if Show then
      PaintSelection;
  End;
  For i:=0 To 7 Do
  begin
    FHandles[i].Visible:=Show and ShowGrabHandles;
//tmp    if not Show then
//      FHandles[i].Parent:=nil;
  end;
  if ControlCount=1 Then Begin
    RepaintControl(Controls[0], False);
  End;
End;

Procedure TRTDesigner.Undo;
var i, n : Integer;
    UndoObj : TUndoObject;
    Ctrl : TControl;
    Reader : TReader;

  Function GetControl(const aName : String;Reader : TReader) : TControl;
  var aParent : TWinControl;
      aClass : String;

    Function LocateControl(const aName : String) : TControl;
    var i : Integer;
        aComp : TComponent;
    Begin
      if BaseControl.Name=aName Then aComp:=BaseControl
      Else aComp:=AssumedOwner.FindComponent(aName);
      if (aComp<>nil) And aComp.Inheritsfrom(TControl) Then Result:=TControl(aComp)
      Else Result:=nil;
      Exit;
      Result:=nil;
      For i:=BaseControl.ControlCount-1 Downto 0 Do if BaseControl.Controls[i].Name=aName Then Begin
        Result:=BaseControl.Controls[i];
        Break;
      End;
    End;

  Begin
    Result:=LocateControl(aName);
    aParent:=TWinControl(LocateControl(Reader.ReadString));
    aClass:=Reader.ReadString;
    if Result=nil Then Begin
      Result:=AddControl(TControlClass(FindClass(aClass)), aParent, 0, 0, aName, True);
    End;
    Result:=TControl(Reader.ReadRootComponent(Result));
    Result.Parent:=aParent;

  End;

  Procedure UndoControl;
  Begin
    AddToSelection(GetControl(UndoObj.Components[i], Reader));
  End;

Begin
  if FUndoList.Count=0 Then Exit;
  n:=FUndoList.Count-1;
  UndoObj:=TUndoObject(FUndoList[n]);
  FReadErrors:=TList.Create;
  Reader:=TReader.Create(UndoObj.Stream, 4096);
  With Reader Do Begin
    OnSetName:=ReaderSetName;
    Root:=BaseControl;
    Owner:=AssumedOwner;
    Parent:=BaseControl;
    Case UndoObj.Kind of
      ukDelete, ukMove, ukSize : Begin
        SelectNone;
        For i:=0 To UndoObj.Components.Count-1 Do
          UndoControl;
        RefreshHandlesPos;
        ShowSelection(ControlCount=1);
      End;
      ukInsert : Begin
        For i:=0 To UndoObj.Components.Count-1 Do Begin
          Ctrl:=GetControl(UndoObj.Components[i], Reader);
          if Ctrl<>nil Then Begin
            RemoveFromSelection(Ctrl);
            Ctrl.Free;
          End;
        End;
        RefreshHandlesPos;
        ShowSelection(ControlCount=1);
      End;
    End;
    Free;
  End;
  For i:=0 To FReadErrors.Count-1 Do if TObject(FReadErrors[i]).InheritsFrom(TControl) Then
    With TControl(FReadErrors[i]) Do Begin
    Left:=Left+GridX;
    Top:=Top+GridY;
  End;
  FReadErrors.Free;
  UndoObj.Free;
  FUndoList.Delete(n);
  if Assigned(FOnUndoChange) Then FOnUndoChange(Self);
End;

Procedure TRTDesigner.UpdateSelection;
var i : Integer;
Begin
  For i:=0 To 7 Do
    FHandles[i].SetPosition;
End;

{ TDesignServer ----------------------------------------------------------------}
Constructor TDesignServer.Create(aOwner : TComponent);
Begin
  inherited Create(aOwner);
  FDesigners:=TList.Create;
  Application.OnMessage:=ApplicationMessage;
  Application.OnActivate:=ActivateApp;
  Application.OnDeactivate:=DeactivateApp;
End;

Destructor TDesignServer.Destroy;
Begin
  Application.OnMessage:=nil;
  FDesigners.Free;
  inherited Destroy;
End;

Procedure TDesignServer.AddDesigner(aDesigner : TRTDesigner);
Begin
  if FDesigners.IndexOf(aDesigner)>=0 Then Exit
  Else FDesigners.Add(aDesigner);
End;

Procedure TDesignServer.ActivateApp(Sender : TObject);
var i : Integer;
Begin
  if Assigned(FOnActivate) Then FOnActivate(Sender);
  For i:=0 To FDesigners.Count-1 Do With TRTDesigner(FDesigners[i]) Do Begin
    if not (Active And EditMode) Then Continue;
    ActivateApp;
  End;
End;

Procedure TDesignServer.ApplicationMessage(var Msg: TMsg; var Handled: Boolean);
var i : Integer;
    aControl : TControl;
Begin
  if Assigned(FOnMessage) Then FOnMessage(Msg, Handled);
  if Handled Then Exit;
  Case Msg.Message of
    WM_KEYFIRST..WM_KEYLAST,
    WM_MOUSEFIRST..WM_MOUSELAST,
    WM_NCMOUSEMOVE, WM_NCLBUTTONDOWN, WM_NCLBUTTONUP,
    WM_PAINT : Begin
      For i:=0 To FDesigners.Count-1 Do With TRTDesigner(FDesigners[i]) Do Begin
        if (MouseStatus<>msNone) Then Begin
          ApplicationMessages(Msg, Handled);
          Exit;
        End;
      End;
      For i:=0 To FDesigners.Count-1 Do With TRTDesigner(FDesigners[i]) Do Begin
        { if the form that the BaseControl is on isn't focused and it should be, focus it! }
        if EditMode And not Active And IsMouseDownMessage(Msg.Message) And (BaseControl<>nil) Then Begin
          aControl:=GetControlUnderMouse;
          if aControl=nil Then Continue;
          if  (aControl = BaseControl) or (aControl.Owner = AssumedOwner) Then { Check if user points
            on something that's located on the form, focus the form }
            SetFocus(BaseControl.Handle);
        End;
        if not (EditMode and Active and BaseControl.Showing) then
          Continue;
        ApplicationMessages(Msg, Handled);
      End;
    End;
  End;
End;

Procedure TDesignServer.DeactivateApp(Sender : TObject);
var i : Integer;
Begin
  if Assigned(FOnDeactivate) Then FOnDeactivate(Sender);
  For i:=0 To FDesigners.Count-1 Do With TRTDesigner(FDesigners[i]) Do Begin
    if not (Active And EditMode) Then Continue;
    DeactivateApp;
  End;
End;

Procedure TDesignServer.RemoveDesigner(aDesigner : TRTDesigner);
var i : Integer;
Begin
  i:=FDesigners.IndexOf(aDesigner);
  if i>=0 Then FDesigners.Delete(i);
End;


{$ifdef ver80}
{$F+}
Procedure CleanUp;
Begin
  DesignServer.Free;
End;
{$F-}
{$endif}


initialization
{$ifdef ver80}
  AddExitProc(CleanUp);
{$endif}
  DesignServer:=TDesignServer.Create(nil);

{$ifndef ver80}
finalization
  DesignServer.Free;
{$endif}


End.

